/* Ekiga -- A VoIP and Video-Conferencing application
 * Copyright (C) 2000-2006 Damien Sandras
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 *
 * Ekiga is licensed under the GPL license and as a special exception,
 * you have permission to link or otherwise combine this program with the
 * programs OPAL, OpenH323 and PWLIB, and distribute the combination,
 * without applying the requirements of the GNU GPL to the OPAL, OpenH323
 * and PWLIB programs, as long as you do follow the requirements of the
 * GNU GPL for all the rest of the software thus combined.
 */


/*
 *                         videooutput_ekiga.cpp -  description
 *                         ----------------------------------
 *   begin                : Sun Nov 17 2006
 *   copyright            : (C) 2006 by Matthias Schneider
 *                          This file is based on videooutput.cpp 
 *                          which is copyright (C) 2000-2006 by Damien Sandras
 *   description          : Stub Class to permit fallback from accelerated 
 *                          video output to classic gdk drawing
 *
 */


#include "../../config.h"

#define P_FORCE_STATIC_PLUGIN 

#include "videooutput_gdk.h"
#ifdef HAS_XV
#include "videooutput_xv.h"
#endif
#include "ekiga.h"

/* Plugin definition */
class PVideoOutputDevice_EKIGA_PluginServiceDescriptor 
: public PDevicePluginServiceDescriptor
{
  public:
    virtual PObject *CreateInstance (int) const 
      {
	return new PVideoOutputDevice_EKIGA (); 
      }

    virtual PStringList GetDeviceNames(int) const 
      { 
	return PStringList("EKIGA"); 
      }

    virtual bool ValidateDeviceName (const PString & deviceName, 
				     int) const 
      { 
	return deviceName.Find("EKIGA") == 0; 
      }
} PVideoOutputDevice_EKIGA_descriptor;

PCREATE_PLUGIN(EKIGA, PVideoOutputDevice, &PVideoOutputDevice_EKIGA_descriptor);


/* The Methods */
PVideoOutputDevice_EKIGA::PVideoOutputDevice_EKIGA ()
{
  outputDevice = NULL;
}

PVideoOutputDevice_EKIGA::~PVideoOutputDevice_EKIGA()
{
  if (outputDevice) delete outputDevice;
}

BOOL 
PVideoOutputDevice_EKIGA::Open (const PString &name,
			      BOOL unused)
{ 
#ifdef HAS_XV
//#if 0
  if (!outputDevice) outputDevice = new PVideoOutputDevice_XV();
  if (name == "EKIGAIN") {
    cached_args.deviceName="XVIN";
    return outputDevice->Open("XVIN", false);
  } else if (name == "EKIGAOUT") {
    cached_args.deviceName="XVOUT";
    return outputDevice->Open("XVOUT", false);
  }
#else
  if (!outputDevice) outputDevice = new PVideoOutputDevice_GDK();
  if (name == "EKIGAIN") {
    cached_args.deviceName="GDKIN";
    return outputDevice->Open("GDKIN", false);
  } else if (name == "EKIGAOUT") {
    cached_args.deviceName="GDKOUT";
    return outputDevice->Open("GDKOUT", false);
  }
#endif
    else if (name == "GDKIN") {
    cached_args.deviceName="GDKIN";
    return outputDevice->Open("GDKIN", false);
  } else if (name == "GDKOUT") {
    cached_args.deviceName="GDKOUT";
    return outputDevice->Open("GDKOUT", false);
  }
}

BOOL PVideoOutputDevice_EKIGA::OpenFull(const PVideoDevice::OpenArgs & args,
                                        BOOL startImmediate)
{
  cached_args = args;
  return PVideoOutputDevice::OpenFull(args, startImmediate);
}

PStringList PVideoOutputDevice_EKIGA::GetDeviceNames() const
{
  PStringList  devlist;
  devlist.AppendString(GetDeviceName());
  return devlist;
}

BOOL PVideoOutputDevice_EKIGA::IsOpen ()
{
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetFrameData (unsigned x,
					   unsigned y,
					   unsigned width,
					   unsigned height,
					   const BYTE * data,
					   BOOL endFrame)
{
#ifdef HAS_XV
  if (!outputDevice->SetFrameData(x,y,width,height,data,endFrame)) {
    // Fallback to GDK
    delete outputDevice;
    outputDevice = new PVideoOutputDevice_GDK();
    if (strcmp (cached_args.deviceName, "XVIN") == 0) cached_args.deviceName="GDKIN";
    if (strcmp (cached_args.deviceName, "XVOUT") == 0) cached_args.deviceName="GDKOUT";
    PVideoOutputDevice::OpenFull(cached_args, TRUE);
    return outputDevice->SetFrameData(x,y,width,height,data,endFrame);
  }
#elif HAS_DX
// to be implemented

#else
  return outputDevice->SetFrameData(x,y,width,height,data,endFrame);
#endif
}

BOOL PVideoOutputDevice_EKIGA::SetVideoFormat(VideoFormat videoFormat)
{
  cached_args.videoFormat = videoFormat;
  if (outputDevice) return outputDevice->SetVideoFormat(videoFormat);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetChannel(int channelNumber)
{
  cached_args.channelNumber = channelNumber;
  if (outputDevice) return outputDevice->SetChannel(channelNumber);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetColourFormat (const PString & colour_format)
{
  cached_args.convertFormat = FALSE;
  cached_args.colourFormat = colour_format;
  if (outputDevice) return outputDevice->SetColourFormat(colour_format);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetColourFormatConverter( const PString & colour_format )
{
  cached_args.convertFormat = TRUE;
  cached_args.colourFormat = colour_format;
  if (outputDevice) return outputDevice->SetColourFormatConverter(colour_format);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetVFlipState( BOOL newVFlipState)
{
  cached_args.flip = newVFlipState;
  if (outputDevice) return outputDevice->SetVFlipState(newVFlipState);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetFrameRate( unsigned rate)
{
  cached_args.rate = rate;
  if (outputDevice) return outputDevice->SetFrameRate(rate);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetFrameSizeConverter(unsigned width, 
                               unsigned height, BOOL bScaleNotCrop)
{
  cached_args.convertSize = TRUE;
  cached_args.width = width;
  cached_args.height = height;
  cached_args.scaleSize = bScaleNotCrop;
  if (outputDevice)
     return outputDevice->SetFrameSizeConverter(width, height, bScaleNotCrop);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetFrameSize(unsigned width, unsigned height) {
  cached_args.convertSize = FALSE;
  cached_args.width = width;
  cached_args.height = height;
  if (outputDevice) return outputDevice->SetFrameSize(width, height);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetBrightness(unsigned newBrightness)
{
  cached_args.brightness = newBrightness;
  if (outputDevice) return outputDevice->SetBrightness(newBrightness);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetWhiteness(unsigned newWhiteness)
{
  cached_args.whiteness = newWhiteness;
  if (outputDevice) return outputDevice->SetWhiteness(newWhiteness);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetColour(unsigned newColour)
{
  cached_args.colour = newColour;
  if (outputDevice) return outputDevice->SetColour(newColour);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetContrast(unsigned newContrast)
{
  cached_args.contrast = newContrast;
  if (outputDevice) return outputDevice->SetContrast(newContrast);
  return TRUE;
}

BOOL PVideoOutputDevice_EKIGA::SetHue(unsigned newHue)
{
  cached_args.hue = newHue;
  if (outputDevice) return outputDevice->SetHue(newHue);
  return TRUE;
}
