
/* Ekiga -- A VoIP and Video-Conferencing application
 * Copyright (C) 2000-2006 Damien Sandras
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 *
 * Ekiga is licensed under the GPL license and as a special exception,
 * you have permission to link or otherwise combine this program with the
 * programs OPAL, OpenH323 and PWLIB, and distribute the combination,
 * without applying the requirements of the GNU GPL to the OPAL, OpenH323
 * and PWLIB programs, as long as you do follow the requirements of the
 * GNU GPL for all the rest of the software thus combined.
 */


/*
 *                         videooutput_ekiga.h  -  description
 *                         ----------------------------
 *   begin                : Sat Feb 17 2001
 *   copyright            : (C) 2000-2006 by Damien Sandras
 *                          This file is based on videooutput.cpp 
 *                          which is copyright (C) 2000-2006 by Damien Sandras
 *   description          : Class to permit to display in EKIGA Drawing Area or
 *                          SDL.
 *
 */


#ifndef _EKIGAVIDEOIO_H_
#define _EKIGAVIDEOIO_H_

#include "common.h"

class GMManager;


class PVideoOutputDevice_EKIGA : public PVideoOutputDevice
{
  PCLASSINFO(PVideoOutputDevice_EKIGA, PVideoOutputDevice);

  public:

  /* DESCRIPTION  :  The constructor.
   * BEHAVIOR     :  /
   * PRE          :  /
   */
  PVideoOutputDevice_EKIGA ();


  /* DESCRIPTION  :  The destructor.
   * BEHAVIOR     :  /
   * PRE          :  /
   */
  ~PVideoOutputDevice_EKIGA ();

  
  /* DESCRIPTION  :  /
   * BEHAVIOR     :  Open the device given the device name.
   * PRE          :  Device name to open, immediately start device.
   */
  BOOL Open (const PString &name,
	     BOOL unused);

  BOOL OpenFull(
    const PVideoDevice::OpenArgs & args,        ///< Parameters to set on opened device
    BOOL startImmediate = TRUE);  ///< Immediately start device

		    		      
  /* DESCRIPTION  :  /
   * BEHAVIOR     :  Return a list of all of the drivers available.
   * PRE          :  /
   */
  PStringList GetDeviceNames() const;


  /* DESCRIPTION  :  /
   * BEHAVIOR     :  Get the maximum frame size in bytes.
   * PRE          :  /
   */
  PINDEX GetMaxFrameBytes() { return 352 * 288 * 3 * 2; }

  
  /* DESCRIPTION  :  /
   * BEHAVIOR     :  Returns TRUE if the output device is open.
   * PRE          :  /
   */
  BOOL IsOpen ();


  /* DESCRIPTION  :  /
   * BEHAVIOR     :  If data for the end frame is received, then we convert
   *                 it from to RGB32 and we display it.
   * PRE          :  x and y positions in the picture (>=0),
   *                 width and height (>=0),
   *                 the data, and a boolean indicating if it is the end
   *                 frame or not.
   */
  BOOL SetFrameData (unsigned x,
		     unsigned y,
		     unsigned width,
		     unsigned height,
		     const BYTE *data,
		     BOOL endFrame);


  /* DESCRIPTION  :  /
   * BEHAVIOR     :  Returns TRUE if the colour format is supported (ie RGB24).
   * PRE          :  /
   */
  BOOL SetColourFormat (const PString &colour_format);


  /* DESCRIPTION  :  /
   * BEHAVIOR     :  Displays the current frame.
   * PRE          :  /
   */
//  BOOL EndFrame();


  /* DESCRIPTION  :  /
   * BEHAVIOR     :  Start displaying.
   * PRE          :  /
   */
  BOOL Start () { if (outputDevice) return outputDevice->Start(); else return TRUE; };
  
  
  /* DESCRIPTION  :  /
   * BEHAVIOR     :  Stop displaying.
   * PRE          :  /
   */
  BOOL Stop () {  if (outputDevice) return outputDevice->Stop();  else return TRUE; };
  
//---------
    BOOL SetVideoFormat(
      VideoFormat videoFormat   ///< New video format
    );

    BOOL SetChannel(
      int channelNumber  ///< New channel number for device.
    );

    BOOL SetColourFormatConverter(
      const PString & colourFormat // New colour format for device.
    );

    BOOL SetVFlipState(
      BOOL newVFlipState    ///< New vertical flip state
    );

    BOOL SetFrameRate(
      unsigned rate  ///< Frames  per second
    );

    BOOL SetFrameSizeConverter(
      unsigned width,        ///< New width of frame
      unsigned height,       ///< New height of frame
      BOOL     bScaleNotCrop ///< Scale or crop/pad preference
    );

    BOOL SetFrameSize(
      unsigned width,   ///< New width of frame
      unsigned height   ///< New height of frame
    );

//----------------
    BOOL SetBrightness(unsigned newBrightness);
    BOOL SetWhiteness(unsigned newWhiteness);
    BOOL SetColour(unsigned newColour);
    BOOL SetContrast(unsigned newContrast);
    BOOL SetHue(unsigned newHue);

 protected:

  static int devices_nbr; /* The number of devices opened */
  int device_id;          /* The current device : encoding or not */

  PVideoOutputDevice* outputDevice;
  PVideoDevice::OpenArgs cached_args;
  enum {REMOTE, LOCAL};
};
#endif
