/* Ekiga -- A VoIP and Video-Conferencing application
 * Copyright (C) 2000-2006 Damien Sandras
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 *
 * Ekiga is licensed under the GPL license and as a special exception,
 * you have permission to link or otherwise combine this program with the
 * programs OPAL, OpenH323 and PWLIB, and distribute the combination,
 * without applying the requirements of the GNU GPL to the OPAL, OpenH323
 * and PWLIB programs, as long as you do follow the requirements of the
 * GNU GPL for all the rest of the software thus combined.
 */


/*
 *                         videooutput_xv.cpp -  description
 *                         ----------------------------------
 *   begin                : Sun Nov 17 2006
 *   copyright            : (C) 2006 by Matthias Schneider
 *                          This file is based on videooutput.cpp 
 *                          which is copyright (C) 2000-2006 by Damien Sandras
 *   description          : Class to allow video output to a XVideo
 *                          accelerated window
 *
 */



#include "../../config.h"

#define P_FORCE_STATIC_PLUGIN 

#include "videooutput_xv.h"
#include "ekiga.h"
#include "misc.h"
#include "main.h"
#include "history.h"


#include "gmconf.h"

#include <ptlib/vconvert.h>

#include <gdk/gdkx.h>

PBYTEArray PVideoOutputDevice_XV::lframeStore;
PBYTEArray PVideoOutputDevice_XV::rframeStore;


int PVideoOutputDevice_XV::devices_nbr = 0;
int PVideoOutputDevice_XV::rf_width = 0;
int PVideoOutputDevice_XV::lf_width = 0;
int PVideoOutputDevice_XV::rf_height = 0;
int PVideoOutputDevice_XV::lf_height = 0;


/* Plugin definition */
class PVideoOutputDevice_XV_PluginServiceDescriptor 
: public PDevicePluginServiceDescriptor
{
  public:
    virtual PObject *CreateInstance (int) const 
      {
	return new PVideoOutputDevice_XV (); 
      }
    
    
    virtual PStringList GetDeviceNames(int) const 
      { 
	return PStringList("XV"); 
      }
    
    virtual bool ValidateDeviceName (const PString & deviceName, 
				     int) const 
      { 
	return deviceName.Find("XV") == 0; 
      }
} PVideoOutputDevice_XV_descriptor;

PCREATE_PLUGIN(XV, PVideoOutputDevice, &PVideoOutputDevice_XV_descriptor);


/* The Methods */
PVideoOutputDevice_XV::PVideoOutputDevice_XV ()
{ 
  is_active = FALSE;
  
  /* Used to distinguish between input and output device. */
  device_id = 0; 

  start_in_fullscreen = FALSE;

  beforeFSSwitch.display = FULLSCREEN;
  beforeFSSwitch.zoom = 1.0;

  lastFrame.display = 99;
  lastFrame.localWidth = 0;
  lastFrame.localHeight = 0;
  lastFrame.remoteWidth = 0;
  lastFrame.remoteHeight = 0;  
  lastFrame.embeddedX = 0;
  lastFrame.embeddedY = 0;
  lastFrame.zoom = 99;

  gnomemeeting_threads_enter ();

  state.localX = 0;
  state.localY = 0;
  state.localWidth = 0;
  state.localHeight = 0;
  state.localOntop = gm_conf_get_bool (VIDEO_DISPLAY_KEY "stay_on_top");
  state.localDecoration = true;

  state.remoteX = 0;
  state.remoteY = 0;
  state.remoteWidth = 0;
  state.remoteHeight = 0;
  state.remoteOntop = gm_conf_get_bool (VIDEO_DISPLAY_KEY "stay_on_top");
  state.remoteDecoration = true;

  /* Change the zoom value following we have to start in fullscreen
   * or not */
  start_in_fullscreen = 
    gm_conf_get_bool (VIDEO_DISPLAY_KEY "start_in_fullscreen");
/*  if (!start_in_fullscreen) {

    if (gm_conf_get_float (VIDEO_DISPLAY_KEY "zoom_factor") == -1.0)
      gm_conf_set_float (VIDEO_DISPLAY_KEY "zoom_factor", 1.00);
  } 
  else {

    gm_conf_set_float (VIDEO_DISPLAY_KEY "zoom_factor", -1.0);
  }*/

  gnomemeeting_threads_leave ();
  lxvWindow = NULL;
  rxvWindow = NULL;

  rDisplay = XOpenDisplay(NULL);
  lDisplay = XOpenDisplay(NULL);
}


PVideoOutputDevice_XV::~PVideoOutputDevice_XV()
{
  PWaitAndSignal m(redraw_mutex);

  if (rxvWindow) {
    rxvWindow->registerSlave(NULL);
    if (lxvWindow) {
      lxvWindow->registerMaster(NULL);
      delete lxvWindow;
    }
    delete rxvWindow;
  }
  if (lDisplay) XCloseDisplay(lDisplay);
  if (rDisplay )XCloseDisplay(rDisplay);

  lframeStore.SetSize (0);
  rframeStore.SetSize (0);

  if (is_active)
    devices_nbr = PMAX (0, devices_nbr-1);
}


BOOL 
PVideoOutputDevice_XV::Open (const PString &name, BOOL unused)
{
  if (name == "XVIN") device_id = 1; 
  return TRUE; 
}

BOOL PVideoOutputDevice_XV::Reinitialize(int display, GtkWidget* main_window, double zoom) 
{
  PTRACE(0, "Reinitializing");
  BOOL ret = FALSE; 
  GtkWidget *history_window = NULL;
  GtkWidget* video = NULL;

  history_window = GnomeMeeting::Process ()->GetHistoryWindow (); 

  if (rxvWindow) rxvWindow->registerSlave(NULL);
  if (lxvWindow) lxvWindow->registerMaster(NULL);

  if (rxvWindow) {
    delete rxvWindow;
    rxvWindow = NULL;
  }

  if (lxvWindow) {
    delete lxvWindow;
    lxvWindow = NULL;
  }

  if (state.localX == 0)       state.localX = rf_width;
  if (state.localWidth == 0)   state.localWidth = lf_width;
  if (state.localHeight == 0)  state.localHeight = lf_height;
  if (state.remoteWidth == 0)  state.remoteWidth = rf_width;
  if (state.remoteHeight == 0) state.remoteHeight = rf_height;

  if (display == REMOTE_VIDEO) {
    video = gm_main_window_get_video_widget (main_window, (int)(rf_width * zoom), (int)(rf_height * zoom),
                                           display);
    if (!GTK_WIDGET_REALIZED(video)) { PTRACE(1, "dropping frame, widget not yet realized"); return 1;}
    rxvWindow = new XVWindow();
    PTRACE(0, "new window REMOTE: "<< (int)(rf_width*zoom)<<"-"<< (int)(rf_height*zoom)<<" and zoom:"<<zoom);
    ret = rxvWindow->init (GDK_DISPLAY(), GDK_WINDOW_XWINDOW(video->window), GDK_GC_XGC(gdk_gc_new( video->window)),
                           video->allocation.x,
                           video->allocation.y,
                           (int)(rf_width * zoom), (int)(rf_height * zoom),
                           rf_width, rf_height); 
    lastFrame.embeddedX=video->allocation.x;
    lastFrame.embeddedY=video->allocation.y;
  }
  else if (display == LOCAL_VIDEO)
  {
    video = gm_main_window_get_video_widget (main_window, (int)(lf_width*zoom), (int)(lf_height*zoom),
                                           display);
    if (!GTK_WIDGET_REALIZED(video)) { PTRACE(1, "dropping frame, widget not yet realized"); return 1;}
    lxvWindow = new XVWindow();
    PTRACE(0, "new window LOCAL: "<< (int)(lf_width*zoom)<<"-"<< (int)(lf_height*zoom)<<" and zoom:"<<zoom);
    ret = lxvWindow->init (GDK_DISPLAY(), GDK_WINDOW_XWINDOW(video->window), GDK_GC_XGC(gdk_gc_new( video->window)),
                           video->allocation.x,
                           video->allocation.y,
                           (int)(lf_width*zoom), (int)(lf_height*zoom),
                           lf_width, lf_height);
    lastFrame.embeddedX=video->allocation.x;
    lastFrame.embeddedY=video->allocation.y;
  }
  else if (display == BOTH)
  {
    gm_main_window_get_video_widget (main_window, GM_QCIF_WIDTH, GM_QCIF_HEIGHT,
                                           display);
    rxvWindow = new XVWindow() ;
    PTRACE(0, "new window BOTH rem: "<< (int)(rf_width)<<"-"<< (int)(rf_height)<<" and zoom:"<<zoom);
    PTRACE(0, "new window BOTH loc: "<< (int)(lf_width)<<"-"<< (int)(lf_height)<<" and zoom:"<<zoom);
    ret = rxvWindow->init (rDisplay, DefaultRootWindow(rDisplay), NULL, 
                           state.remoteX, state.remoteY,
                           state.remoteWidth, state.remoteHeight,
                           rf_width, rf_height);
    if (ret)
    {
      lxvWindow = new XVWindow();
      if (!lxvWindow->init (lDisplay, DefaultRootWindow(lDisplay), NULL,
                            state.localX, state.localY,
                            state.localWidth, state.localHeight,
                            lf_width, lf_height))
      {
        PTRACE(1,"Unable to open local video window");
        gm_history_window_insert(history_window, _("Unable to open local video window"));
      }
    }
  }
  else if (display == FULLSCREEN)
  {
    gm_main_window_get_video_widget (main_window, GM_QCIF_WIDTH, GM_QCIF_HEIGHT,
                                           display);
    rxvWindow = new XVWindow();
    PTRACE(0, "new window fullscreen rem: "<< (int)(rf_width)<<"-"<< (int)(rf_height)<<" and zoom:"<<zoom);
    PTRACE(0, "new window FULLSCREEN loc: "<< (int)(lf_width)<<"-"<< (int)(lf_height)<<" and zoom:"<<zoom);
    ret = rxvWindow->init (rDisplay, DefaultRootWindow(rDisplay), NULL,
                           0, 0,
                           rf_width, rf_height,
                           rf_width, rf_height);
    lxvWindow = new XVWindow();
    if (!lxvWindow->init (lDisplay, rxvWindow->getWindow(), NULL,
                          0, 0,
                          lf_width, lf_height,
                          lf_width, lf_height))
    {
      PTRACE(1,"Unable to open Picture-in-Picture");
      gm_history_window_insert(history_window, _("Unable to open Picture-in-Picture"));
    }
    if (rxvWindow) rxvWindow->registerSlave(lxvWindow);
    if (lxvWindow) lxvWindow->registerMaster(rxvWindow);
    if (rxvWindow) rxvWindow->toggleFullscreen();
  }
  else if (display == BOTH_INCRUSTED)
  {
    video = gm_main_window_get_video_widget (main_window, (int)(rf_width*zoom), (int)(rf_height*zoom),
                                           display);
    if (!GTK_WIDGET_REALIZED(video)) { PTRACE(1, "dropping frame, widget not yet realized"); return 1;}
    rxvWindow = new XVWindow();
    PTRACE(0, "new window BOTH_INCRUSTED rem: "<< (int)(rf_width*zoom)+1<<"-"<< (int)(rf_height*zoom)+1<<" and zoom:"<<zoom);
    PTRACE(0, "new window BOTH_INCRUSTED loc: "<< (int)(rf_width*zoom/3)<<"-"<< (int)(rf_height*zoom/3)<<" and zoom:"<<zoom);
    ret = rxvWindow->init (GDK_DISPLAY(), GDK_WINDOW_XWINDOW(video->window), GDK_GC_XGC(gdk_gc_new( video->window)),
                           video->allocation.x,
                           video->allocation.y,
                           (int)(rf_width*zoom), (int)(rf_height*zoom),
                           rf_width, rf_height);
    lxvWindow = new XVWindow();
    if (!lxvWindow->init (GDK_DISPLAY(), rxvWindow->getWindow(),GDK_GC_XGC(gdk_gc_new( video->window)),
                          (int)(rf_width*zoom*2/3), (int)(rf_height*zoom*2/3), 
                          (int)(rf_width*zoom/3), (int)(rf_height*zoom/3),
                          lf_width, lf_height))
    {
      PTRACE(1,"Unable to open Picture-in-Picture");
      gm_history_window_insert(history_window, _("Unable to open Picture-in-Picture"));
    }

    if (rxvWindow) rxvWindow->registerSlave(lxvWindow);
    if (lxvWindow) lxvWindow->registerMaster(rxvWindow);
    lastFrame.embeddedX=video->allocation.x;
    lastFrame.embeddedY=video->allocation.y;
  }
  else if (display == BOTH_INCRUSTED_WINDOW)
  {
    gm_main_window_get_video_widget (main_window, GM_QCIF_WIDTH, GM_QCIF_HEIGHT,
                                           display);
    rxvWindow = new XVWindow();
    PTRACE(0, "new window BOTH_INCRUSTED_WINDOW rem: "<< (int)(rf_width)<<"-"<< (int)(rf_height)<<" and zoom:"<<zoom);
    PTRACE(0, "new window BOTH_INCRUSTED_WINDOW loc: "<< (int)(lf_width)<<"-"<< (int)(lf_height)<<" and zoom:"<<zoom);
    ret = rxvWindow->init (rDisplay, DefaultRootWindow(rDisplay), NULL,
                           state.remoteX, state.remoteY,
                           state.remoteWidth,state.remoteHeight,
                           rf_width, rf_height);
    lxvWindow = new XVWindow();
    if (!lxvWindow->init (lDisplay, rxvWindow->getWindow(), NULL,
                          0, 0,
                          lf_width, lf_height,
                          lf_width, lf_height))
    {
      PTRACE(1,"Unable to open Picture-in-Picture");
      gm_history_window_insert(history_window, _("Unable to open Picture-in-Picture"));
    }
    if (rxvWindow) rxvWindow->registerSlave(lxvWindow);
    if (lxvWindow) lxvWindow->registerMaster(rxvWindow);
  }

  if (video) PTRACE(0, "widget size: "<< video->allocation.width<<"x"<<video->allocation.height);
  if (video) PTRACE(0, "widget position: "<< video->allocation.x<<","<<video->allocation.y);

  lastFrame.display = display;
  lastFrame.localWidth  = lf_width;
  lastFrame.localHeight = lf_height;
  lastFrame.remoteWidth  = rf_width;
  lastFrame.remoteHeight = rf_height;

  if (display!=FULLSCREEN) 
  {
    if (rxvWindow && state.remoteOntop) rxvWindow->toggleOntop();
    if (rxvWindow && (!state.remoteDecoration)) rxvWindow->toggleDecoration();
    if (lxvWindow && state.localOntop) lxvWindow->toggleOntop();
    if (lxvWindow && (!state.localDecoration)) lxvWindow->toggleDecoration();
  }

  if (!ret)
  {
    PTRACE(1,"Unable to open primary XVideo window - falling back to software rendering");
    gm_history_window_insert(history_window, _("Unable to open primary XVideo window - falling back to software rendering"));
  }
  return ret;
}

void PVideoOutputDevice_XV::UpdateState(int display) 
{
  switch (display)
  {
    case BOTH:
      if (lxvWindow)
      {
        lxvWindow->getWindow(&state.localX, &state.localY, &state.localWidth, &state.localHeight);
        state.localOntop = lxvWindow->isOntop();
        state.localDecoration = lxvWindow->hasDecoration();
      }
    case BOTH_INCRUSTED_WINDOW:
      if (rxvWindow)
      {
        rxvWindow->getWindow(&state.remoteX, &state.remoteY, &state.remoteWidth, &state.remoteHeight);
        state.remoteOntop = rxvWindow->isOntop();
        state.remoteDecoration = rxvWindow->hasDecoration();
      }
  }
}

BOOL PVideoOutputDevice_XV::UpdateWindows ()
{
  GtkWidget *main_window = NULL;
  int display = LOCAL_VIDEO;
  double zoom = 1.0;

  main_window = GnomeMeeting::Process ()->GetMainWindow (); 
  display = gm_conf_get_int (VIDEO_DISPLAY_KEY "video_view");
  zoom = gm_conf_get_float (VIDEO_DISPLAY_KEY "zoom_factor");

  /* If there is only one device open, ignore the setting, and 
   * display what we can actually display.
   */
  if (devices_nbr <= 1) {
    if (device_id == REMOTE)
      display = REMOTE_VIDEO;
    else if (device_id == LOCAL)
      display = LOCAL_VIDEO;
  }

  if ((zoom == -1.0) && (beforeFSSwitch.display!=FULLSCREEN))
  {
    if ((rxvWindow) && (!rxvWindow->isFullScreen()))
    {
      PTRACE(0, "switching back from FS because XVWindows is no longer in FS");
      display = beforeFSSwitch.display;
      lastFrame.display = beforeFSSwitch.display;
      beforeFSSwitch.display = FULLSCREEN;
      zoom = beforeFSSwitch.zoom;
      gm_conf_set_float (VIDEO_DISPLAY_KEY "zoom_factor", zoom);
      gm_mw_zooms_menu_update_sensitivity (main_window, zoom);
      lastFrame.zoom = beforeFSSwitch.zoom;
      return(Reinitialize(display, main_window, zoom));
    }
    return true;
  }

  if ((zoom == -1.0) && (beforeFSSwitch.display==FULLSCREEN) && (devices_nbr > 1)) {
    PTRACE(0,"switching to FS mode");
    // do FS
    beforeFSSwitch.display = display;
    display = FULLSCREEN;
    UpdateState(lastFrame.display);
    //lastFrame.display = FULLSCREEN;
    zoom = 1.0;
    lastFrame.zoom = 1.0;
    return (Reinitialize(display, main_window, zoom));
  } 

  if ((zoom != -1.0) && (beforeFSSwitch.display!=FULLSCREEN) && (devices_nbr > 1)) {
    PTRACE(0, "switching back from FS mode (shouldnt happen this way");
    display = beforeFSSwitch.display;
    lastFrame.display = beforeFSSwitch.display;
    beforeFSSwitch.display = FULLSCREEN;
    zoom = beforeFSSwitch.zoom;
    gm_conf_set_float (VIDEO_DISPLAY_KEY "zoom_factor", zoom);
    gm_mw_zooms_menu_update_sensitivity (main_window, zoom);
    lastFrame.zoom = beforeFSSwitch.zoom;
    return(Reinitialize(display, main_window, zoom));
  }

  if (zoom != 0.0 && zoom != 0.5 && zoom != 1.00 && zoom != 2.00) zoom = 1.0;
  beforeFSSwitch.zoom = zoom;

  if ((lastFrame.localWidth != lf_width) ||
      (lastFrame.localHeight != lf_height) ||
      (lastFrame.remoteWidth != rf_width) ||
      (lastFrame.remoteHeight != rf_height))
  {
    PTRACE(0, "Resolution of frames have changed");
    lastFrame.zoom = zoom;
    return(Reinitialize(display, main_window, zoom));
  }

  if (display != lastFrame.display)
  {
    PTRACE(0, "Display mode has changed");
    UpdateState(lastFrame.display);
    lastFrame.zoom = zoom;
    return(Reinitialize(display, main_window, zoom));
  }

  if  ((display != BOTH) &&
      (display != BOTH_INCRUSTED_WINDOW) &&
      (display != FULLSCREEN))
  {
    GtkWidget* video = gm_main_window_get_video_widget (main_window, (int)(rf_width*zoom), 
                                          (int)(rf_height*zoom), display);
    if ((lastFrame.zoom != zoom) ||
        (lastFrame.embeddedX!=video->allocation.x) ||
        (lastFrame.embeddedY!=video->allocation.y)) 
    {
      PTRACE(0, "Zoom or Location has changed");
      if (rxvWindow) {
         rxvWindow->setWindow(video->allocation.x,
                              video->allocation.y,
                              (int)(rf_width*zoom),
                              (int)(rf_height*zoom));
      }

      if (lxvWindow) {
        if (display==BOTH_INCRUSTED)
             lxvWindow->setWindow((int)(rf_width*zoom*2/3), (int)(rf_height*zoom*2/3), (int)(rf_width*zoom/3), (int)(rf_height*zoom/3));
           else 
         {
           lxvWindow->setWindow(video->allocation.x,
                                video->allocation.y,
                                (int)(lf_width*zoom),
                                (int)(lf_height*zoom));
        }
      }
      lastFrame.zoom=zoom;
      lastFrame.embeddedX=video->allocation.x;
      lastFrame.embeddedY=video->allocation.y;
      return (true);
    }
  }
  return (true);
}

BOOL PVideoOutputDevice_XV::Redraw ()
{
  // for fallback testing
  return FALSE;
  BOOL ret = TRUE; 

  /* Take the mutexes before the redraw */
  redraw_mutex.Wait ();
  /* Device is now open */
  if (!is_active) {
    is_active = TRUE;
    devices_nbr = PMIN (2, devices_nbr+1);
  }

  gnomemeeting_threads_enter ();

  if (device_id==LOCAL) ret = UpdateWindows();

  if ((rxvWindow)  && (rframeStore.GetPointer())) {
    rxvWindow->putFrame((uint8_t*)rframeStore.GetPointer(), rf_width, rf_height);
  }
  if ((lxvWindow) && (lframeStore.GetPointer())) {
    lxvWindow->putFrame((uint8_t*)lframeStore.GetPointer(), lf_width, lf_height);
  }
  gnomemeeting_threads_leave ();
  redraw_mutex.Signal ();

  return ret;
}


PStringList PVideoOutputDevice_XV::GetDeviceNames() const
{
  PStringList  devlist;
  devlist.AppendString(GetDeviceName());

  return devlist;
}


BOOL PVideoOutputDevice_XV::IsOpen ()
{
  return TRUE;
}


BOOL PVideoOutputDevice_XV::SetFrameData (unsigned x,
					   unsigned y,
					   unsigned width,
					   unsigned height,
					   const BYTE * data,
					   BOOL endFrame)
{
  if (x+width > width || y+height > height)
    return FALSE;

  if (!endFrame)
    return FALSE;

  if (device_id == LOCAL) {
    lframeStore.SetSize ((int)(width * height * 3 / 2));
    lf_width = width;
    lf_height = height;

    if (converter)
      converter->Convert (data, lframeStore.GetPointer ());
     else 
      memcpy (lframeStore.GetPointer(), data,(int) (width * height * 3 / 2)); // YUV420P -> YUV420P
  }
  else 
  {
    rframeStore.SetSize ((int)(width * height * 3 / 2));
    rf_width = width;
    rf_height = height;

    if (converter)
      converter->Convert (data, rframeStore.GetPointer ());
     else 
      memcpy (rframeStore.GetPointer(), data, (int)(width * height * 3 / 2));  // YUV420P -> YUV420P
  }

  return (Redraw ());
}


BOOL PVideoOutputDevice_XV::EndFrame()
{
  return Redraw ();
}


BOOL PVideoOutputDevice_XV::SetColourFormat (const PString & colour_format)
{
  if (colour_format == "YUV420P")
    return PVideoOutputDevice::SetColourFormat (colour_format);

  return FALSE;  
}
